#!/usr/bin/perl -w  # -*- cperl -*- #
#
#  playlist.pm - A simple module to generate a playlist file.
#
#  GNU MP3D - A portable(ish) MP3 server.
#
# Homepage:
#   http://www.gnump3d.org/
#
# Author:
#  Steve Kemp <steve@steve.org.uk>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Steve Kemp
#  ---
#  http://www.steve.org.uk/
#
#


package gnump3d::playlist;  # must live in playlist.pm
use gnump3d::files;
require Exporter;
use strict;
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);


# set the version for version checking
my $VERSION = '$Revision: 1.1.1.1 $';


@ISA         = qw(Exporter);
@EXPORT      = qw( &playlistForDirectory );
%EXPORT_TAGS = ( );


# your exported package globals go here,
# as well as any optionally exported functions
@EXPORT_OK   = ( ) ;
my %HASH     = ();



#
# fisher_yates_shuffle( \@array ) : generate a random permutation
# of @array in place
#
#  Taken from the Perl Cookbook.
#
sub fisher_yates_shuffle (@)
{
    my $array = shift;
    my $i;
    for ($i = @$array; --$i; )
    {
        my $j = int rand ($i+1);
        next if $i == $j;
        @$array[$i,$j] = @$array[$j,$i];
    }
}





#
#  Generate a playlist for a given directory.
#  This is uber-simple
#
sub playlistForDirectory( $ $ $  )
{
    my ( $dir, $recursive, $random ) = (@_);
    my @files = ();

    #
    # Read in the files for the playlist.
    #
    if ( $recursive )
    {
	@files = &filesInDirRecursively( $dir );
    }
    else
    {
	@files = &filesInDir( $dir );
    }

    if ( $random )
    {
        # permutes @array in place
	fisher_yates_shuffle( \@files );
    }


    #
    # Make sure that each file we include is an audio file.
    #
    my @results;
    foreach my $entry (@files )
    {
	next if ( ! isAudio( $entry ) );
	push @results, $entry;
    }

    return( join( "\n", @results ) );
}



#
# End of module
#
1;
